@@ -37,6 +37,23 @@ class dicom_properties:
         self.ri = 0.0
         self.image_shape = []
 
+    def read_dicom_slop_intercept(self, slice):
+        if Tag(0x28, 0x1052) in slice:
+            self.ri = slice[0x28, 0x1052].value #Rescale Intercept
+            self.rs = slice[0x28, 0x1053].value #Rescale Slope
+        elif Tag(0x11, 0x103b) in slice:
+            self.rs = slice[0x11, 0x103b].value #Pixel Scale
+            self.ri = slice[0x11, 0x103c].value #Pixel Offset
+        elif Tag(0x40, 0x9096) in slice and Tag(0x40, 0x9224) in slice[0x40, 0x9096][0]:
+            self.ri = slice[0x40, 0x9096][0][0x40, 0x9224].value #Rescale Intercept
+            self.rs = slice[0x40, 0x9096][0][0x40, 0x9225].value #Rescale Slope
+        elif Tag(0x3004, 0x000e) in slice:
+            self.rs = slice[0x3004, 0x000e].value #Rescale Slope
+        if hasattr(self.ri, '__len__'):
+            self.ri = self.ri[0]
+        if hasattr(self.rs, '__len__'):
+            self.rs = self.rs[0]
+
     def read_dicom_properties(self, slice, nextSlice=None):
         # pixel aspects, assuming all slices are the same
 
@@ -68,29 +85,33 @@ class dicom_properties:
         if self.io == "" or self.io == None:
             self.io = [1.0, 0.0, 0.0, 0.0, 1.0, 0.0]
         #orientation = [io[0], io[1], io[2], io[3], io[4], io[5]]
-        if Tag(0x28, 0x1052) in slice:
-            self.ri = slice[0x28, 0x1052].value #Rescale Intercept
-            self.rs = slice[0x28, 0x1053].value #Rescale Slope
-        elif Tag(0x11, 0x103b) in slice:
-            self.rs = slice[0x11, 0x103b].value #Pixel Scale
-            self.ri = slice[0x11, 0x103c].value #Pixel Offset
-        elif Tag(0x40, 0x9096) in slice and Tag(0x40, 0x9224) in slice[0x40, 0x9096][0]:
-            self.ri = slice[0x40, 0x9096][0][0x40, 0x9224].value #Rescale Intercept
-            self.rs = slice[0x40, 0x9096][0][0x40, 0x9225].value #Rescale Slope
-        elif Tag(0x3004, 0x000e) in slice:
-            self.rs = slice[0x3004, 0x000e].value #Rescale Slope
-        if hasattr(self.ri, '__len__'):
-            self.ri = self.ri[0]
-        if hasattr(self.rs, '__len__'):
-            self.rs = self.rs[0]
+        self.read_dicom_slop_intercept(slice)
+
 
         self.img_shape = list(slice.pixel_array.shape)
 
+def separate_series(dicomFiles):
+    """
+    Read dicom and return a dictionary with the different series separated
+    """
+    files = {}
+    #Load dicom files
+    for file in dicomFiles:
+        try:
+            seriesInstanceUID = pydicom.read_file(file)[0x0020, 0x000e].value
+        except pydicom.errors.InvalidDicomError:
+            ds = pydicom.read_file(file, force=True)
+            ds.file_meta.TransferSyntaxUID = pydicom.uid.ImplicitVRLittleEndian
+            seriesInstanceUID = ds[0x0020, 0x000e].value
+        if seriesInstanceUID not in files:
+            files[seriesInstanceUID] = []
+        files[seriesInstanceUID].append(file)
+    return files
 
 def read_dicom(dicomFiles):
     """
 
-    Read dicom files and return an float 3D image
+    Read dicom files and return a float 3D image
     """
     files = []
     #Load dicom files
@@ -107,7 +128,7 @@ def read_dicom(dicomFiles):
     skipcount = 0
     if len(files) > 1:
         for f in files:
-            if hasattr(f, 'SliceLocation'):
+            if hasattr(f, 'SliceLocation') or hasattr(f, 'ImagePositionPatient'):
                 slices.append(f)
             else:
                 skipcount = skipcount + 1
@@ -115,8 +136,16 @@ def read_dicom(dicomFiles):
         if skipcount >0:
             logger.info("skipped, no SliceLocation: {}".format(skipcount))
 
+        #Remove images with same SopInstanceUID
+        noDuplicateSlices = []
+        tmpSopInstanceUID = []
+        for slice in slices:
+            if not slice[0x0008, 0x0018].value in tmpSopInstanceUID:
+                noDuplicateSlices.append(slice)
+                tmpSopInstanceUID.append(slice[0x0008, 0x0018].value)
+
         # ensure they are in the correct order. Sort according Image Position along z
-        slices = sorted(slices, key=lambda s: s[0x0020, 0x0032][2])
+        slices = sorted(noDuplicateSlices, key=lambda s: s[0x0020, 0x0032][2])
     else:
         logger.error('no file available')
         return
@@ -139,8 +168,9 @@ def read_dicom(dicomFiles):
     # fill 3D array with the images from the files
     for i, s in enumerate(slices):
         img2d = s.pixel_array
-        img3d[i, :, :] = img2d
-    img3d = dicomProperties.rs*img3d+dicomProperties.ri
+        dicomPropertiesSlice = dicom_properties()
+        dicomPropertiesSlice.read_dicom_slop_intercept(s)
+        img3d[i, :, :] = dicomPropertiesSlice.rs*img2d+dicomPropertiesSlice.ri
 
     img_result = itk.image_from_array(np.float32(img3d))
     img_result.SetSpacing(dicomProperties.spacing)
@@ -161,7 +191,7 @@ def read_dicom(dicomFiles):
 def read_3d_dicom(dicomFile, flip=False):
     """
 
-    Read dicom file and return an float 3D image
+    Read dicom file and return a float 3D image
     """
     files = []
     try:
@@ -171,7 +201,6 @@ def read_3d_dicom(dicomFile, flip=False):
         ds.file_meta.TransferSyntaxUID = pydicom.uid.ImplicitVRLittleEndian
         files.append(ds)
 
-    # skip files with no SliceLocation (eg scout views)
     slices = []
     if len(files) == 1:
         slices.append(files[0])
@@ -222,7 +251,7 @@ def read_3d_dicom(dicomFile, flip=False):
 
 def image_convert(inputImage, pixeltype=None):
     """
-    Compute relative statistical uncertainty 
+    Convert the pixelType of the image
 
     - inputImage: input itk image to convert
     - pixeltype: string representing the type of the output pixel:
@@ -239,7 +268,8 @@ def image_convert(inputImage, pixeltype=None):
         OutputType = itk.Image[itk.ctype(pixeltype), inputImage.GetImageDimension()]
         castFilter = itk.CastImageFilter[InputType, OutputType].New()
         castFilter.SetInput(inputImage)
-        return castFilter
+        castFilter.Update()
+        return castFilter.GetOutput()
 
     return inputImage
 
@@ -302,7 +332,7 @@ class Test_Convert(LoggedTestCase):
         shutil.rmtree(tmpdirpath)
     def test_convert_rtDose(self):
         tmpdirpath = tempfile.mkdtemp()
-        filenameRTDose = wget.download("https://github.com/OpenGATE/GateTools/raw/master/dataTest/rtdose.dcm", out=tmpdirpath, bar=None)
+        filenameRTDose = wget.download("https://gitlab.in2p3.fr/opengate/gatetools_data/-/raw/master/rtdose.dcm?inline=false", out=tmpdirpath, bar=None)
         convertedImage = read_3d_dicom([os.path.join(tmpdirpath, filenameRTDose)])
         itk.imwrite(convertedImage, os.path.join(tmpdirpath, "testConvert.mha"))
         with open(os.path.join(tmpdirpath, "testConvert.mha"),"rb") as fnew:
@@ -312,7 +342,7 @@ class Test_Convert(LoggedTestCase):
         shutil.rmtree(tmpdirpath)
     def test_convert_dicom(self):
         tmpdirpath = tempfile.mkdtemp()
-        filename = wget.download("https://github.com/OpenGATE/GateTools/blob/master/dataTest/tomoSPECT.dcm?raw=true", out=tmpdirpath, bar=None)
+        filename = wget.download("https://gitlab.in2p3.fr/opengate/gatetools_data/-/raw/master/tomoSPECT.dcm?inline=false", out=tmpdirpath, bar=None)
         convertedImage = read_3d_dicom([os.path.join(tmpdirpath, filename)])
         itk.imwrite(convertedImage, os.path.join(tmpdirpath, "testConvert.mha"))
         with open(os.path.join(tmpdirpath, "testConvert.mha"),"rb") as fnew:
@@ -322,7 +352,7 @@ class Test_Convert(LoggedTestCase):
         shutil.rmtree(tmpdirpath)
     def test_convert_dicom_flip(self):
         tmpdirpath = tempfile.mkdtemp()
-        filename = wget.download("https://github.com/OpenGATE/GateTools/blob/master/dataTest/tomoSPECT.dcm?raw=true", out=tmpdirpath, bar=None)
+        filename = wget.download("https://gitlab.in2p3.fr/opengate/gatetools_data/-/raw/master/tomoSPECT.dcm?inline=false", out=tmpdirpath, bar=None)
         convertedImage = read_3d_dicom([os.path.join(tmpdirpath, filename)], flip=True)
         itk.imwrite(convertedImage, os.path.join(tmpdirpath, "testConvert.mha"))
         with open(os.path.join(tmpdirpath, "testConvert.mha"),"rb") as fnew:
