@@ -6,24 +6,23 @@
 # -----------------------------------------------------------------------------
 
 import numpy as np
-import sys
 import os
-import uproot3 as uproot
-import time
 import tokenize
 from io import BytesIO
 from matplotlib import pyplot as plt
-from matplotlib.colors import LogNorm
+import uproot
 import logging
-logger=logging.getLogger(__name__)
+
+logger = logging.getLogger(__name__)
+
 
 # -----------------------------------------------------------------------------
 def load(filename, nmax=-1, shuffle=False):
-    ''' 
+    """ 
     Load a PHSP (Phase-Space) file
     Output is numpy structured array
-    '''
-    
+    """
+
     b, extension = os.path.splitext(filename)
     nmax = int(nmax)
 
@@ -40,67 +39,65 @@ def load(filename, nmax=-1, shuffle=False):
         return load_npy(filename, nmax, shuffle)
 
     logger.error('dont know how to open phsp with extension ',
-          extension,
-          ' (known extensions: .root .npy)')
+                 extension,
+                 ' (known extensions: .root .npy)')
     exit(0)
 
 
 # -----------------------------------------------------------------------------
 def load_root(filename, nmax=-1):
-    '''
+    """
     Load a PHSP (Phase-Space) file in root format
     Output is numpy structured array
-    '''
-    
+    """
+
     nmax = int(nmax)
     # Check if file exist
-    if (not os.path.isfile(filename)):
-        logger.error("File '"+filename+"' does not exist.")
+    if not os.path.isfile(filename):
+        logger.error("File '" + filename + "' does not exist.")
         exit()
 
     # Check if this is a root file
     try:
-        f = uproot.open(filename)
+        with uproot.open(filename) as f:
+            k = f.keys()
+            try:
+                psf = f['PhaseSpace']
+            except Exception:
+                logger.error("This root file does not look like a PhaseSpace, keys are: ",
+                              f.keys(), ' while expecting "PhaseSpace"')
+                exit()
+
+            # Get keys
+            names = [k for k in psf.keys()]
+            n = psf.num_entries
+
+            # Convert to arrays (this take times)
+            if nmax != -1:
+                a = psf.arrays(entry_stop=nmax, library="numpy")
+            else:
+                a = psf.arrays(library="numpy")
+
+            # Concat arrays
+            d = np.column_stack([a[k] for k in psf.keys()])
+            # d = np.float64(d) # slow
     except Exception:
-        logger.error("File '"+filename+"' cannot be opened, not root file ?")
+        logger.error("File '" + filename + "' cannot be opened, not root file ?")
         exit()
 
-    # Look for a single key named "PhaseSpace"
-    k = f.keys()
-    try:
-        psf = f['PhaseSpace']
-    except Exception:
-        logger.error("This root file does not look like a PhaseSpace, keys are: ",
-              f.keys(), ' while expecting "PhaseSpace"')
-        exit()
-        
-    # Get keys
-    names = [k.decode('UTF-8') for k in psf.keys()]
-    n = psf.numentries
-
-    # Convert to arrays (this take times)
-    if (nmax != -1):
-        a = psf.arrays(entrystop=nmax)
-    else:
-        a = psf.arrays()
-
-    # Concat arrays
-    d = np.column_stack([a[k] for k in psf.keys()])
-    #d = np.float64(d) # long
-    
     return d, names, n
 
 
 # -----------------------------------------------------------------------------
 def load_npy(filename, nmax=-1, shuffle=False):
-    '''
+    """
     Load a PHSP (Phase-Space) file in npy
     Output is numpy structured array
-    '''
-    
+    """
+
     # Check if file exist
-    if (not os.path.isfile(filename)):
-        logger.error("File '"+filename+"' does not exist.")
+    if not os.path.isfile(filename):
+        logger.error("File '" + filename + "' does not exist.")
         exit()
 
     x = np.load(filename, mmap_mode='r')
@@ -112,20 +109,19 @@ def load_npy(filename, nmax=-1, shuffle=False):
             x = x[:nmax]
 
     data = x.view(np.float32).reshape(x.shape + (-1,))
-    #data = np.float64(data) # long
+    # data = np.float64(data) # slow
     return data, list(x.dtype.names), n
 
 
-
 # -----------------------------------------------------------------------------
 def humansize(nbytes):
-    '''
+    """
     https://stackoverflow.com/questions/14996453/python-libraries-to-calculate-human-readable-filesize-from-bytes
-    '''
+    """
 
     suffixes = ['B', 'KB', 'MB', 'GB', 'TB', 'PB']
     i = 0
-    while nbytes >= 1024 and i < len(suffixes)-1:
+    while nbytes >= 1024 and i < len(suffixes) - 1:
         nbytes /= 1024.
         i += 1
     f = ('%.2f' % nbytes).rstrip('0').rstrip('.')
@@ -134,34 +130,34 @@ def humansize(nbytes):
 
 # -----------------------------------------------------------------------------
 def save_npy(filename, data, keys):
-    '''
+    """
     Write a PHSP (Phase-Space) file in npy
-    '''
+    """
 
     dtype = []
     for k in keys:
         dtype.append((k, 'f4'))
-    
+
     r = np.zeros(len(data), dtype=dtype)
     i = 0
     for k in keys:
-        r[k] = data[:,i]
-        i = i+1
+        r[k] = data[:, i]
+        i = i + 1
 
     np.save(filename, r)
 
 
 # -----------------------------------------------------------------------------
 def remove_keys(data, keys, rm_keys):
-    '''
+    """
     Remove som keys
-    '''
+    """
 
     cols = np.arange(len(keys))
     index = []
     if len(rm_keys) == 0:
         return data, keys
-    
+
     for k in rm_keys:
         if k not in keys:
             logger.error('Error the key', k, 'does not exist in', keys)
@@ -177,11 +173,11 @@ def remove_keys(data, keys, rm_keys):
 
 # -----------------------------------------------------------------------------
 def str_keys_to_array_keys(keys):
-    '''
+    """
     Convert string of keys to arrays of key
-    '''
+    """
 
-    if keys == None:
+    if keys is None:
         return []
     dd = tokenize.tokenize(BytesIO(keys.encode('utf-8')).readline)
     keys = []
@@ -193,9 +189,9 @@ def str_keys_to_array_keys(keys):
 
 # -----------------------------------------------------------------------------
 def get_E(data, keys):
-    '''
-    Retrive the E from a dataset
-    '''
+    """
+    Retrieve the E from a dataset
+    """
 
     try:
         Ei = keys.index('Ekine')
@@ -204,15 +200,15 @@ def get_E(data, keys):
             Ei = keys.index('E')
         except:
             raise RuntimeError("Error, cannot find key 'Ekine' nor 'E'. Keys are: ", keys)
-    E = data[:,Ei]
+    E = data[:, Ei]
     return E, Ei
 
 
 # -----------------------------------------------------------------------------
 def fig_get_sub_fig(ax, i):
-    '''
-    Retrive a fig nb 
-    '''
+    """
+    Retrieve a fig nb
+    """
 
     # check if single fig
     if not type(ax) is np.ndarray:
@@ -229,41 +225,41 @@ def fig_get_sub_fig(ax, i):
 
 # -----------------------------------------------------------------------------
 def fig_get_nb_row_col(nfig):
-    '''
+    """
     Compute a fig with adapted row/col for n fig
-    '''
-    
+    """
+
     nrow = int(np.sqrt(nfig))
-    ncol = int(nfig/nrow)
-    if ncol*nrow<nfig:
+    ncol = int(nfig / nrow)
+    if ncol * nrow < nfig:
         nrow += 1
     return nrow, ncol
 
 
 # -----------------------------------------------------------------------------
 def fig_rm_empty_plot(nfig, ax):
-    '''
+    """
     Remove empty plot
-    '''
+    """
 
     nrow, ncol = fig_get_nb_row_col(nfig)
-    r = nrow-1
+    r = nrow - 1
     i = nfig
-    while i<ncol*nrow:
-        c = i-int(i/ncol)*ncol
-        ax[r,c].set_axis_off()
-        i = i+1  
+    while i < ncol * nrow:
+        c = i - int(i / ncol) * ncol
+        ax[r, c].set_axis_off()
+        i = i + 1
+
+    # -----------------------------------------------------------------------------
 
 
-
-# -----------------------------------------------------------------------------
 def keys_toggle_angle(keys):
-    '''
+    """
     In the list of keys, toggle angleXY to XY or XY to angleXY
-    '''
+    """
 
     k = keys.copy()
-    if 'X' in keys and 'Y' in keys:        
+    if 'X' in keys and 'Y' in keys:
         k.remove('X')
         k.remove('Y')
         k.append('angleXY')
@@ -272,17 +268,15 @@ def keys_toggle_angle(keys):
             k.append('X')
             k.append('Y')
             k.remove('angleXY')
-    return k   
+    return k
+
 
-        
 # -----------------------------------------------------------------------------
 def select_keys(data, input_keys, output_keys):
-    '''
+    """
     Keep only the given keys
-    '''
+    """
 
-    cols = np.arange(len(input_keys))
-    index = []
     if len(output_keys) == 0:
         logger.error('Error, select_keys is void')
         exit(0)
@@ -293,7 +287,7 @@ def select_keys(data, input_keys, output_keys):
             i = input_keys.index(k)
             cols.append(i)
         except:
-            logger.error('Error, cannot find',k,'in keys:',input_keys)
+            logger.error('Error, cannot find', k, 'in keys:', input_keys)
             exit(0)
 
     data = data[:, cols]
@@ -302,29 +296,28 @@ def select_keys(data, input_keys, output_keys):
 
 # -----------------------------------------------------------------------------
 def add_angle(data, keys, k1='X', k2='Y'):
-    '''
+    """
     Add and compute angleXY in the list of keys
     angle = atan2(k2,k1)
-    '''
+    """
 
     if 'angleXY' in keys:
         return data, keys
-    
+
     i1 = keys.index(k1)
     i2 = keys.index(k2)
-    angle = np.arctan2(data[:,i2], data[:,i1])
+    angle = np.arctan2(data[:, i2], data[:, i1])
     data = np.column_stack((data, angle))
     k = keys.copy()
-    k.append('angleXY')    
+    k.append('angleXY')
     return data, k
- 
 
 
 # -----------------------------------------------------------------------------
 def add_vector_angle(data, keys, radius, k='angleXY', k1='X', k2='Y'):
-    '''
+    """
     Add X and Y from angleXY
-    '''
+    """
 
     # nothing to do if already exist
     if k1 in keys and k2 in keys:
@@ -334,7 +327,7 @@ def add_vector_angle(data, keys, radius, k='angleXY', k1='X', k2='Y'):
         logger.warning('Cannot convert angle, the key angleXY does not exist in ', keys)
 
     i = keys.index(k)
-    angle = data[:,i]
+    angle = data[:, i]
     dx = radius * np.cos(angle)
     dy = radius * np.sin(angle)
     data = np.column_stack((data, dx))
@@ -345,11 +338,12 @@ def add_vector_angle(data, keys, radius, k='angleXY', k1='X', k2='Y'):
     kk.append(k2)
     return data, kk
 
+
 # -----------------------------------------------------------------------------
 def add_missing_angle(data, input_keys, output_keys, radius):
-    '''
+    """
     Add missing keys (angleXY or X+Y)
-    '''
+    """
 
     if 'angleXY' in output_keys:
         data, input_keys = add_angle(data, input_keys)
@@ -362,27 +356,28 @@ def add_missing_angle(data, input_keys, output_keys, radius):
 
 # -----------------------------------------------------------------------------
 def fig_histo2D(ax, data, keys, k, nbins, color='g'):
-    '''
+    """
     Fig 2D histo
-    '''
-    
+    """
+
     i1 = keys.index(k[0])
-    x = data[:,i1]
+    x = data[:, i1]
     i2 = keys.index(k[1])
-    y = data[:,i2]
+    y = data[:, i2]
     if color == 'g':
         cmap = plt.cm.Greens
     if color == 'r':
         cmap = plt.cm.Reds
     if color == 'b':
         cmap = plt.cm.Blues
-        
+
     counts, xedges, yedges, im = ax.hist2d(x, y, bins=(nbins, nbins), alpha=1, cmap=cmap)
-    #, norm=LogNorm())
+    # , norm=LogNorm())
     plt.colorbar(im, ax=ax)
     ax.set_xlabel(k[0])
     ax.set_ylabel(k[1])
 
+
 #####################################################################################
 import unittest
 import hashlib
@@ -391,30 +386,34 @@ import tempfile
 import shutil
 from gatetools.logging_conf import LoggedTestCase
 
+
 class Test_Phsp(LoggedTestCase):
     def test_phsp_convert(self):
         logger.info('Test_Phsp test_phsp_convert')
         tmpdirpath = tempfile.mkdtemp()
-        filenameRoot = wget.download("https://github.com/OpenGATE/GateTools/raw/master/dataTest/phsp.root", out=tmpdirpath, bar=None)
+        filenameRoot = wget.download(
+            "https://gitlab.in2p3.fr/opengate/gatetools_data/-/raw/master/phsp.root?inline=false", out=tmpdirpath,
+            bar=None)
         data, read_keys, m = load(os.path.join(tmpdirpath, filenameRoot), -1)
         save_npy(os.path.join(tmpdirpath, "testphsp.npy"), data, read_keys)
-        with open(os.path.join(tmpdirpath, "testphsp.npy"),"rb") as fnew:
+        with open(os.path.join(tmpdirpath, "testphsp.npy"), "rb") as fnew:
             bytesNew = fnew.read()
             new_hash = hashlib.sha256(bytesNew).hexdigest()
             self.assertTrue("cec796ec5764d039b02e15d504e80ccf2b2c35e0e5380985245262faf0ff0892" == new_hash)
         dataNPY, read_keysNPY, mNPY = load(os.path.join(tmpdirpath, "testphsp.npy"), -1)
-        self.assertTrue(np.allclose(131.69868, np.amax(dataNPY[:,2])))
+        self.assertTrue(np.allclose(131.69868, np.amax(dataNPY[:, 2])))
         shutil.rmtree(tmpdirpath)
 
     def test_phsp_info(self):
         logger.info('Test_Phsp test_phsp_info')
         tmpdirpath = tempfile.mkdtemp()
-        filenameRoot = wget.download("https://github.com/OpenGATE/GateTools/raw/master/dataTest/phsp.root", out=tmpdirpath, bar=None)
+        filenameRoot = wget.download(
+            "https://gitlab.in2p3.fr/opengate/gatetools_data/-/raw/master/phsp.root?inline=false", out=tmpdirpath,
+            bar=None)
         data, read_keys, m = load(os.path.join(tmpdirpath, filenameRoot), -1)
         self.assertTrue("17.27 MB" == humansize(os.stat(os.path.join(tmpdirpath, filenameRoot)).st_size))
         self.assertTrue(np.float32 == data.dtype)
         self.assertTrue(782127 == m)
         self.assertTrue(7 == len(read_keys))
-        self.assertTrue(np.allclose(131.69868, np.amax(data[:,2])))
+        self.assertTrue(np.allclose(131.69868, np.amax(data[:, 2])))
         shutil.rmtree(tmpdirpath)
-
