@@ -156,7 +156,7 @@ def enclosed_area(vs):
     a = 0
     x0,y0 = vs[0]
     logger.debug("x0={} y0={}".format(x0,y0))
-    for [x1,y1] in vs[1:]:
+    for [x1,y1] in np.vstack((vs, vs[0]))[1:]: # NK: calculate also segment from last to first vertex. If not, area is underestimated.
         dx = x1-x0
         dy = y1-y0
         a += 0.5*(y0*dx - x0*dy)
@@ -386,7 +386,7 @@ class region_of_interest(object):
         if len(dz) == 1:
             self.dz = dz.pop()
         else:
-            dz = set(np.diff(np.around(self.zlist,decimals=6)))
+            dz = set(np.around(np.diff(self.zlist),decimals=6))
             if len(dz) == 1:
                 self.dz = dz.pop()
             elif len(dz) == 0:
@@ -422,7 +422,7 @@ class region_of_interest(object):
         if len(dz) == 1:
             self.dz = dz.pop()
         else:
-            dz = set(np.diff(np.around(zlist,decimals=6)))
+            dz = set(np.around(np.diff(zlist),decimals=6))
             if len(dz) == 1:
                 self.dz = dz.pop()
             else:
@@ -777,12 +777,12 @@ from .logging_conf import LoggedTestCase
 class Test_ROI(LoggedTestCase):
     def test_roi(self):
         tmpdirpath = tempfile.mkdtemp()
-        filenameStruct = wget.download("https://github.com/OpenGATE/GateTools/raw/master/dataTest/rtstruct.dcm", out=tmpdirpath, bar=None)
+        filenameStruct = wget.download("https://gitlab.in2p3.fr/opengate/gatetools_data/-/raw/master/rtstruct.dcm?inline=false", out=tmpdirpath, bar=None)
         structset = pydicom.read_file(os.path.join(tmpdirpath, filenameStruct))
 
         # roi names
         roi_names = list_roinames(structset)
-        filenameCT = wget.download("https://github.com/OpenGATE/GateTools/raw/master/dataTest/ct.mha", out=tmpdirpath, bar=None)
+        filenameCT = wget.download("https://gitlab.in2p3.fr/opengate/gatetools_data/-/raw/master/ct.mha?inline=false", out=tmpdirpath, bar=None)
         img = itk.imread(os.path.join(tmpdirpath, filenameCT))
         self.assertTrue(len(roi_names) == 11)
         self.assertTrue(roi_names[0] == 'External')
